import click, os
from .source import virtual, folders, files

@click.group()
def cherrypie():
    pass

# new_path = r'c:\Users\limzy\OneDrive\Desktop\Projects\trial'
dest_path = os.getcwd()


@cherrypie.command()
@click.option('-n', '--name', type=click.STRING, default='venv', show_default=True, help='The name of the virtual environment')
@click.option('-a', '--activate', type=click.Choice(['y', 'n']), required= False, default='y', show_default=True, help='Allow virtual environment to be activated.')
@click.argument('path', type=click.Path(exists=True), default=os.getcwd())
    
def paint(name, path, activate):
    try:
        virtual.create_venv(name, path)
        click.echo("Success!!")
        if activate == 'y':
            virtual.activate_venv(name, path)
    except Exception as e:
        click.echo(f'I need virtual succour: {e}')
    
    try:
        virtual.install_depend(path=path, name=name)
    except Exception as e:
        click.echo(f'I need virtual succour: {e}')
        
        
        
map_dict = {
    'root': [
        '.env',
        '.gitignore',
        'requirements.txt',
        {
            'app': [
                {
                    'routers': ['route.py']
                },
                'config.py',
                'db.py',
                'main.py',
                'models.py',
                'schemas.py',
                'utils.py',
            ]
        },
        {
            'scripts': ['script1.py']
        },
        {
            'tests': ['test_example.py']
        }
    ]
}

def unpacker(dictio, path):
    for k, v in dictio.items():
        curr_path = path
        if k != 'root':
            curr_path = os.path.join(path, k)
        for content in v:
            if isinstance(content, dict):
                unpacker(content, curr_path)
            else:
               files.create_file(content, curr_path)
    return "Created!"
    
        

@cherrypie.command()
def blossom():
    try:
        directories = ['scripts', 'tests']
        for fol in directories:
            folders.folder(fol, dest_path)
        # folders.folder('scripts')
        folders.create_subfolder(dest_path, 'app', 'routers')
        click.echo("All done!!")
    except Exception as e:
        click.echo(f'I need folder succour: {str(e)}')   
    
    # Create files
    try:
        unpacker(map_dict, dest_path)
    except Exception as e:
        click.echo(f'I need files succour: {str(e)}')
        
if __name__=="__main__":
    cherrypie()


import click, subprocess, os
path = os.getcwd()


def folder(name, path):
    path = os.path.join(path, name)
    try:
        result = subprocess.run(['mkdir', path], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        if result.returncode == 0:
            click.echo('Command run successfully.')
            click.echo(result.stdout)
        else:
            click.echo(f'Command failed with error: {result.stderr}')
    except Exception as e:
        click.echo(f'An error occured: {str(e)}')


def create_subfolder(path, folder_name, subfolder_name):
    """
    Create a subfolder within a specified directory.
    """
    try:
        # Combine the provided directory and subfolder names to create the full path
        subfolder_path = os.path.join(path, folder_name, subfolder_name)

        # Use os.makedirs to create the subfolder (including any necessary parent folders)
        os.makedirs(subfolder_path)

        click.echo(f"Subfolder '{subfolder_name}' created in '{folder_name}'.")
    except Exception as e:
        click.echo(f"An error occurred: {str(e)}")

if __name__ == '__main__':
    folder()

    


import os, subprocess, click

def create_venv(name, path):
    """Creates a python virtual environment.

    Args:
        name (_str_): This will be the name of your virtual environment.
    """
    try:
        subprocess.run(['virtualenv', '-p', 'python3', rf'{path}\{name}'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        click.echo(f'Successfully created virtual environment: {name}')
    except Exception as e:
        click.echo(f'Failed to create virtual environment. Error: {str(e)}')
    
    # try:
    #     subprocess.run(['pip', 'install', '-r', rf'C:\Users\limzy\OneDrive\Desktop\Projects\cherry\utils\requirements.txt.txt', '--target', f'{path}\{name}\Lib\site-packages'])
    #     click.echo(f'Successfully installed all the required dependencies.')
    # except Exception as e:
    #     click.echo(f'Failure!!! Failure!!: {e}')



    
def activate_venv(name, path):
    try:
        activation_script_path = os.path.join(rf'{path}', f'{name}', 'Scripts', 'activate.bat')
        click.echo(activation_script_path)
        subprocess.run([activation_script_path], check=True, shell=(os.name=='nt'))
        click.echo(f'Successfully activated virtual environment: {name}')
    except Exception as e:
        click.echo(f'Failed to activate virtual environment. Error: {str(e)}')

def install_depend(path, name):
    try:
        subprocess.run(['pip', 'install', '-r', rf'{os.path.dirname(__file__)}\utils\requirements.txt.txt', '--target', f'{path}\{name}\Lib\site-packages'])
        click.echo(f'Successfully installed all the required dependencies.')
    except Exception as e:
        click.echo(f'Failure!!! Failure!!: {e}')
    


import click, os, subprocess

folder_path = os.path.dirname(__file__)

# @click.command()
# @click.argument('name', type=click.STRING, required=True)
# @click.argument('path', type=click.Path(exists=True))
def create_file(name, path):

    try:
        if not os.path.exists(path):
            raise click.BadParameter(f'{path} does not exist')
        
        # if not name.endswith(".py"):
        #     name += ".py"
        
        new_folder_path = os.path.join(path, f'{name}')
        
        txt_file = rf'{folder_path}\utils\{name}.txt'
        py_file = rf'{new_folder_path}'
        
        res = subprocess.run(['echo', '', '>', new_folder_path], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        
        with open(txt_file, 'r') as t, open(py_file, 'w') as p:
            txt_contents = t.read()
            p.write(txt_contents)
            
        if res.returncode == 0:
            click.echo(f"Successfully created {name} in the {new_folder_path} directory")
            
        else:
            click.echo(res.stderr)
    except Exception as e:
        click.echo(f'Ran an error: {str(e)}')
    
@click.command()
# @click.argument('content', type=click.STRING, metavar='CONTENT')
def edit_file():
    txt_file = rf'{folder_path}\imports.txt'
    py_file = rf'{folder_path}\app\all.py'
    try:
        with open(txt_file, 'r') as t, open(py_file, 'w') as p:
            txt_contents = t.read()
            p.write(txt_contents)
            # click.echo("Content copied successfully")
    except Exception as e:
        click.echo(f'Failed: {str(e)}')
    
    
if __name__=="__main__":
    edit_file()